package com.zsc.edu.gateway.modules.notice.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zsc.edu.gateway.framework.security.UserDetailsImpl;
import com.zsc.edu.gateway.modules.notice.dto.UserMessageDto;
import com.zsc.edu.gateway.modules.notice.query.AdminMessageQuery;
import com.zsc.edu.gateway.modules.notice.query.UserMessageQuery;
import com.zsc.edu.gateway.modules.notice.service.UserMessageService;
import com.zsc.edu.gateway.modules.notice.vo.AdminMessageVo;
import com.zsc.edu.gateway.modules.notice.vo.UserMessageVo;
import lombok.AllArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 用户消息Controller
 *
 * @author harry_yao
 */
@AllArgsConstructor
@RestController
@RequestMapping("api/rest/message")
public class UserMessageController {

    private final UserMessageService service;

    /**
     * 普通用户查看消息详情
     *
     * @param userDetails 操作用户
     * @param messageId   消息ID
     * @return 用户消息详情
     */
    @GetMapping("/self/{messageId}")
    public UserMessageVo selfDetail(@AuthenticationPrincipal UserDetailsImpl userDetails, @PathVariable("messageId") Long messageId) {
        return service.detail(messageId, userDetails.getId());
    }

    /**
     * 普通用户分页查询消息，不能设置查询参数userId和username
     *
     * @param userDetails 操作用户
     * @param query       查询参数
     * @return 分页数据
     */
    @GetMapping("/self")
    public IPage<UserMessageVo> selfPage(Page<UserMessageVo> pageParam, @AuthenticationPrincipal UserDetailsImpl userDetails, UserMessageQuery query) {
        query.userId = userDetails.id;
        query.name = null;
        return service.page(pageParam, query);
    }

    /**
     * 普通用户统计自己未读消息数量
     *
     * @param userDetails 操作用户
     * @return 数据
     */
    @GetMapping("/countUnread")
    public int countUnread(@AuthenticationPrincipal UserDetailsImpl userDetails) {
        return service.countUnread(userDetails);
    }

    /**
     * 普通用户确认消息已读，如果提交的已读消息ID集合为空，则将所有未读消息设为已读
     *
     * @param userDetails 操作用户
     * @param messageIds  已读消息ID集合
     * @return 确认已读数量
     */
    @PatchMapping("/read")
    public int acknowledge(@AuthenticationPrincipal UserDetailsImpl userDetails, @RequestBody List<Long> messageIds) {
        return service.markAsRead(userDetails, messageIds);
    }

    /**
     * 管理查询消息详情
     *
     * @param messageId 消息ID
     * @return 用户消息详情
     */
    @GetMapping("/{userId}/{messageId}")
    @PreAuthorize("hasAuthority('MESSAGE_QUERY')")
    public UserMessageVo detail(@PathVariable("userId") Long userId, @PathVariable("messageId") Long messageId) {
        return service.detail(userId, messageId);
    }

    /**
     * 管理员分页查询消息
     *
     * @param query 查询参数
     * @return 分页数据
     */
    @GetMapping
    @PreAuthorize("hasAuthority('MESSAGE_QUERY')")
    public IPage<AdminMessageVo> page(Page<UserMessageVo> page, AdminMessageQuery query) {
        return service.getAdminMessagePage(page, query);
    }

    /**
     * 管理员手动创建消息
     *
     * @param dto 表单数据
     * @return 消息列表
     */
    @PostMapping
    @PreAuthorize("hasAuthority('MESSAGE_CREATE')")
    public Boolean create(@RequestBody UserMessageDto dto) {
        return service.createByAdmin(dto);
    }
}
