package com.zsc.edu.gateway.modules.notice.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.framework.message.email.EmailSender;
import com.zsc.edu.gateway.framework.message.sms.SmsSender;
import com.zsc.edu.gateway.framework.security.UserDetailsImpl;
import com.zsc.edu.gateway.modules.notice.dto.UserMessageDto;
import com.zsc.edu.gateway.modules.notice.entity.*;
import com.zsc.edu.gateway.modules.notice.mapper.MessageMapper;
import com.zsc.edu.gateway.modules.notice.query.AdminMessageQuery;
import com.zsc.edu.gateway.modules.notice.query.UserMessageQuery;
import com.zsc.edu.gateway.modules.notice.repo.MessageRepository;
import com.zsc.edu.gateway.modules.notice.repo.UserMessageRepository;
import com.zsc.edu.gateway.modules.notice.service.UserMessageService;
import com.zsc.edu.gateway.modules.notice.vo.AdminMessageVo;
import com.zsc.edu.gateway.modules.notice.vo.UserMessageVo;
import com.zsc.edu.gateway.modules.system.entity.User;
import com.zsc.edu.gateway.modules.system.repo.UserRepository;
import lombok.AllArgsConstructor;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;


/**
 * 用户消息Service
 *
 * @author harry_yao
 */
@AllArgsConstructor
@Service
public class UserMessageServiceImpl extends ServiceImpl<UserMessageRepository, UserMessage> implements UserMessageService {

    private final MessageRepository messageRepo;
    private final EmailSender emailSender;
    private final SmsSender smsSender;
    private final UserMessageRepository userMessageRepository;
    private final UserRepository userRepository;
    private final MessageMapper messageMapper;
    /**
     * 查询消息详情
     *
     * @param messageId 消息ID
     * @param userId    用户ID
     * @return 查询详情
     */

    @Override
    public UserMessageVo detail(Long messageId, Long userId) {
        if (userMessageRepository.selectByMessageIdAndUserId(messageId, userId) == null) {
            UserMessage userMessage = new UserMessage();
            userMessage.setUserId(userId);
            userMessage.setMessageId(messageId);
            userMessage.setIsRead(true);
            userMessage.setReadTime(LocalDateTime.now());
            save(userMessage);
        }
        return userMessageRepository.selectByMessageIdAndUserId(messageId, userId);
    }

    /**
     * 分页查询用户消息
     *
     * @param query 查询表单
     * @param page  分页参数
     * @return 页面数据
     */
    @Override
    public IPage<UserMessageVo> page(Page<UserMessageVo> page, UserMessageQuery query) {
        return userMessageRepository.page(page, query);
    }

    /**
     * 统计用户未读消息数量
     *
     * @param userDetails 操作用户
     * @return 未读消息数量
     */
    @Override
    public Integer countUnread(UserDetailsImpl userDetails) {
        QueryWrapper<UserMessage> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userDetails.getId()).eq("is_read", true);
        return Math.toIntExact(userMessageRepository.selectCount(queryWrapper));
    }

    /**
     * 设为已读
     *
     * @param userDetails 操作用户
     * @param messageIds  消息ID集合，如为空，则将该用户的所有未读消息设为已读
     * @return 修改记录数量
     */
    @Override
    public boolean markAsRead(UserDetailsImpl userDetails, List<Long> messageIds) {
        if (CollectionUtils.isEmpty(messageIds)) {
            throw new RuntimeException("messageIds is NULL!");
        }
        return this.lambdaUpdate().eq(UserMessage::getUserId, userDetails.getId())
                .in(UserMessage::getMessageId, messageIds)
                .set(UserMessage::getIsRead, true)
                .set(UserMessage::getReadTime, LocalDateTime.now())
                .update();
    }


    /**
     * 管理员查询消息列表
     *
     * @return 消息设置列表
     */
    @Override
    public IPage<AdminMessageVo> getAdminMessagePage(Page<UserMessageVo> page, AdminMessageQuery query) {
        return userMessageRepository.pageAdmin(page, query);
    }


    /**
     * 管理员手动创建用户消息并发送
     *
     * @param dto 表单数据
     * @return 创建的用户消息列表
     */
    @Transactional
    @Override
    public Boolean createByAdmin(UserMessageDto dto) {
        Set<User> users = new HashSet<>(userRepository.selectList(new LambdaQueryWrapper<User>().in(User::getId, dto.userIds)));
        Message message = messageMapper.toEntity(dto);
        messageRepo.insert(message);
        Set<UserMessage> userMessages = users.stream()
                .map(user -> new UserMessage(null, user.getId(), message.getId(), true, null))
                .collect(Collectors.toSet());
        send(users, message);
        return saveBatch(userMessages);
    }


    /**
     * 以邮件、短信等形式发送消息，只有非html格式的消息才能以短信方式发送
     *
     * @param users   接收者
     * @param message 消息
     */
    @Async
    void send(Set<User> users, Message message) {
        if (message.email) {
            emailSender.send(users.stream().map(User::getEmail).collect(Collectors.toSet()), message);
        }
        if (message.sms && !message.html) {
            smsSender.send(users.stream().map(User::getPhone).collect(Collectors.toSet()), message.content);
        }
    }


    /**
     * 系统自动创建用户消息并发送
     *
     * @param receivers 接收者
     * @param payload   消息内容
     */
    @Transactional
    public Boolean createBySystem(Set<User> receivers, MessagePayload payload) {
        AtomicBoolean email = new AtomicBoolean(false);
        AtomicBoolean sms = new AtomicBoolean(false);
        Optional.of(messageRepo.selectById(payload.type)).ifPresent(message -> {
            email.set(message.email);
            sms.set(message.sms);
        });
        Message message = new Message(payload.type, true, email.get(), sms.get(),
                payload.html, payload.type.name(), payload.content);
        messageRepo.insert(message);
        Set<UserMessage> userMessages = receivers.stream().map(user ->
                new UserMessage(null, user.getId(), message.getId(), true, null)).collect(Collectors.toSet());
        send(receivers, message);
        return saveBatch(userMessages);
    }

}
